"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _camelizeStyleName = require("fbjs/lib/camelizeStyleName");

var _camelizeStyleName2 = _interopRequireDefault(_camelizeStyleName);

var _rule = require("./rule");

var _rule2 = _interopRequireDefault(_rule);

var _MediaQuery = require("../models/MediaQuery");

var _MediaQuery2 = _interopRequireDefault(_MediaQuery);

var _RuleSet = require("../models/RuleSet");

var _RuleSet2 = _interopRequireDefault(_RuleSet);

var _NestedSelector = require("../models/NestedSelector");

var _NestedSelector2 = _interopRequireDefault(_NestedSelector);

var _ValidRuleSetChild = require("../models/ValidRuleSetChild");

var _ValidRuleSetChild2 = _interopRequireDefault(_ValidRuleSetChild);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var declaration = /^\s*([\w-]+):\s*([^;]*);\s*$/;
var startNesting = /^\s*([\w\.#:&>~+][^{]+?)\s*\{\s*$/;
var startMedia = /^\s*@media\s+([^{]+?)\s*\{\s*$/;
var stopNestingOrMedia = /^\s*}\s*$/;

/* This is a bit complicated.
*  Basically, you get an array of strings and an array of interpolations.
*  You want to interleave them, so that they're processed together.
*  That's easy enough.
*  Except you also want to split the strings by line.
*  Still ok I guess.
*  Except that some of the interpolations are within a line i.e. background: ${ colors.white };
*  So short-circuit those by converting to a string and concatenating so later
*  when you split by "\n" you get valid lines of CSS.
*  I know, right?
*
*  Anyway, this needs to be replaced by a real CSS parser. TODO: that.
* */
var interleave = function interleave(strings, interpolations) {
  var linesAndInterpolations = strings[0].split('\n');
  interpolations.forEach(function (interp, i) {
    /* Complex, Rule-based interpolation (could be multi-line, or nesting etc) */
    if (interp instanceof _ValidRuleSetChild2.default) {
      linesAndInterpolations.push(interp);
      if (strings[i + 1]) linesAndInterpolations.push.apply(linesAndInterpolations, _toConsumableArray(strings[i + 1].split('\n')));
    } else {
      /* Simple (value) interpolation. Concatenate and move on. */
      var lastStr = linesAndInterpolations.pop();
      linesAndInterpolations.push.apply(linesAndInterpolations, _toConsumableArray((lastStr + interp + (strings[i + 1] || '')).split('\n')));
    }
  });
  return linesAndInterpolations;
};

exports.default = function (strings) {
  for (var _len = arguments.length, interpolations = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    interpolations[_key - 1] = arguments[_key];
  }

  /* A data structure we can use to traverse the CSS */
  var currentLevel = {
    parent: null,
    ruleSet: new _RuleSet2.default()
  };
  var linesAndInterpolations = interleave(strings, interpolations);

  var processLine = function processLine(line) {
    var _ref = startNesting.exec(line) || [];

    var _ref2 = _slicedToArray(_ref, 2);

    var _ = _ref2[0];
    var subSelector = _ref2[1];

    var _ref3 = declaration.exec(line) || [];

    var _ref4 = _slicedToArray(_ref3, 3);

    var __ = _ref4[0];
    var property = _ref4[1];
    var value = _ref4[2];

    var _ref5 = startMedia.exec(line) || [];

    var _ref6 = _slicedToArray(_ref5, 2);

    var ___ = _ref6[0];
    var mediaQuery = _ref6[1];

    var popNestingOrMedia = stopNestingOrMedia.exec(line);

    /* ARE WE STARTING A NESTING? */
    if (subSelector) {
      var subRules = new _RuleSet2.default();
      var nesting = new _NestedSelector2.default(subSelector, subRules);
      currentLevel.ruleSet.add(nesting);
      currentLevel = {
        parent: currentLevel,
        ruleSet: subRules
      };

      /* ARE WE STARTING A MEDIA QUERY? */
    } else if (mediaQuery) {
      var _subRules = new _RuleSet2.default();
      var media = new _MediaQuery2.default(mediaQuery, _subRules);
      currentLevel.ruleSet.add(media);
      currentLevel = {
        parent: currentLevel,
        ruleSet: _subRules
      };

      /* ARE WE A NORMAL RULE? */
    } else if (property && value) {
      var newRule = (0, _rule2.default)((0, _camelizeStyleName2.default)(property), value);
      currentLevel.ruleSet.add(newRule);
    } else if (popNestingOrMedia) {
      if (!currentLevel.parent) {
        console.error(linesAndInterpolations);
        console.error(currentLevel);
        throw new Error("CSS Syntax Error — Trying to un-nest one too many times");
      }
      currentLevel = currentLevel.parent;
    }
  };

  var processLineOrInterp = function processLineOrInterp(lineOrInterp) {
    if (typeof lineOrInterp === 'string') {
      processLine(lineOrInterp);
    } else if (lineOrInterp instanceof _ValidRuleSetChild2.default) {
      currentLevel.ruleSet.add(lineOrInterp);
    } else {
      console.warn("I don't know what to do with this:", lineOrInterp);
    }
  };

  linesAndInterpolations.forEach(processLineOrInterp);
  return currentLevel.ruleSet;
};

module.exports = exports["default"];