'use strict';

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _toggle = require('../toggle');

var _toggle2 = _interopRequireDefault(_toggle);

var _concat = require('../concat');

var _concat2 = _interopRequireDefault(_concat);

var _rule = require('../rule');

var _rule2 = _interopRequireDefault(_rule);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('toggle', function () {
  var defaultRule = (0, _rule2.default)('default', 'rule');
  var optionA = (0, _rule2.default)('option', 'a');
  var optionB = (0, _rule2.default)('option', 'b');

  it('should handle an empty case', function () {
    var namespace = (0, _toggle2.default)('namespace', {});
    (0, _expect2.default)(namespace()).toEqual((0, _concat2.default)());
    (0, _expect2.default)(namespace('')).toEqual((0, _concat2.default)());
    (0, _expect2.default)(namespace(' ')).toEqual((0, _concat2.default)());
  });

  it('should always inject default even if no input passed', function () {
    var namespace = (0, _toggle2.default)('namespace', {
      default: defaultRule
    });
    (0, _expect2.default)(namespace()).toEqual((0, _concat2.default)(defaultRule));
    (0, _expect2.default)(namespace('')).toEqual((0, _concat2.default)(defaultRule));
    (0, _expect2.default)(namespace(' ')).toEqual((0, _concat2.default)(defaultRule));
  });

  it('should handle any combination of inputs', function () {
    var namespace = (0, _toggle2.default)('namespace', {
      default: defaultRule,
      a: optionA,
      b: optionB
    });
    (0, _expect2.default)(namespace()).toEqual((0, _concat2.default)(defaultRule));
    (0, _expect2.default)(namespace('a')).toEqual((0, _concat2.default)(defaultRule, optionA));
    (0, _expect2.default)(namespace('a b')).toEqual((0, _concat2.default)(defaultRule, optionA, optionB));
    (0, _expect2.default)(namespace('b a')).toEqual((0, _concat2.default)(defaultRule, optionB, optionA));
    (0, _expect2.default)(namespace('b')).toEqual((0, _concat2.default)(defaultRule, optionB));
  });

  it('should be ok without a default', function () {
    var namespace = (0, _toggle2.default)('namespace', {
      a: optionA,
      b: optionB
    });
    (0, _expect2.default)(namespace()).toEqual((0, _concat2.default)());
    (0, _expect2.default)(namespace('a')).toEqual((0, _concat2.default)(optionA));
    (0, _expect2.default)(namespace('a b')).toEqual((0, _concat2.default)(optionA, optionB));
    (0, _expect2.default)(namespace('b a')).toEqual((0, _concat2.default)(optionB, optionA));
    (0, _expect2.default)(namespace('b')).toEqual((0, _concat2.default)(optionB));
  });

  it('should throw if passed something not present', function () {
    var namespace = (0, _toggle2.default)('namespace', {
      a: optionA
    });
    (0, _expect2.default)(function () {
      return namespace('b');
    }).toThrow("namespace: Unknown value 'b'. Valid values are:\na");
  });

  it('should not mention the default as a valid', function () {
    var namespace = (0, _toggle2.default)('namespace', {
      default: defaultRule,
      a: optionA
    });
    (0, _expect2.default)(function () {
      return namespace('b');
    }).toThrow("namespace: Unknown value 'b'. Valid values are:\na");
  });
});