'use strict';

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _MediaQuery = require('./MediaQuery');

var _MediaQuery2 = _interopRequireDefault(_MediaQuery);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('MediaQuery', function () {
  describe('#fullQuery', function () {
    it('should add brackets to a simple example', function () {
      var mediaQuery = new _MediaQuery2.default('min-width: 500px');
      (0, _expect2.default)(mediaQuery.fullQuery()).toEqual('@media (min-width: 500px)');
    });
    it('should not add brackets to a simple example with brackets', function () {
      var mediaQuery = new _MediaQuery2.default('(min-width: 500px)');
      (0, _expect2.default)(mediaQuery.fullQuery()).toEqual('@media (min-width: 500px)');
    });
    it('should add brackets to a weird double example', function () {
      var mediaQuery = new _MediaQuery2.default('min-width: 500px) and (max-width: 1000px');
      (0, _expect2.default)(mediaQuery.fullQuery()).toEqual('@media (min-width: 500px) and (max-width: 1000px)');
    });
    it('should not add brackets to a normal double example', function () {
      var mediaQuery = new _MediaQuery2.default('(min-width: 500px) and (max-width: 1000px)');
      (0, _expect2.default)(mediaQuery.fullQuery()).toEqual('@media (min-width: 500px) and (max-width: 1000px)');
    });
    it('should be ok if screen is used', function () {
      var mediaQuery = new _MediaQuery2.default('screen and (min-width: 500px)');
      (0, _expect2.default)(mediaQuery.fullQuery()).toEqual('@media screen and (min-width: 500px)');
    });
    it('should be ok if print is used', function () {
      var mediaQuery = new _MediaQuery2.default('print and (min-width: 500px)');
      (0, _expect2.default)(mediaQuery.fullQuery()).toEqual('@media print and (min-width: 500px)');
    });
    it("should happily return invalid syntax if that's what you give it", function () {
      var mediaQuery = new _MediaQuery2.default('screen and min-width: 500px');
      (0, _expect2.default)(mediaQuery.fullQuery()).toEqual('@media (screen and min-width: 500px)');
    });
  });
});