'use strict';

var _RuleSet = require('../RuleSet');

var _RuleSet2 = _interopRequireDefault(_RuleSet);

var _Rule = require('../Rule');

var _Rule2 = _interopRequireDefault(_Rule);

var _MediaQuery = require('../MediaQuery');

var _MediaQuery2 = _interopRequireDefault(_MediaQuery);

var _NestedSelector = require('../NestedSelector');

var _NestedSelector2 = _interopRequireDefault(_NestedSelector);

var _ValidRuleSetChild = require('../ValidRuleSetChild');

var _ValidRuleSetChild2 = _interopRequireDefault(_ValidRuleSetChild);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('RuleSet', function () {
	describe('add', function () {
		it('should add a rule', function () {
			var rule = new _ValidRuleSetChild2.default();
			var ruleSet = new _RuleSet2.default();
			ruleSet.add(rule);
			(0, _expect2.default)(ruleSet.rules).toEqual([rule]);
		});

		it('should add multiple rules', function () {
			var rule1 = new _ValidRuleSetChild2.default();
			var rule2 = new _ValidRuleSetChild2.default();
			var ruleSet = new _RuleSet2.default();
			ruleSet.add(rule1, rule2);
			(0, _expect2.default)(ruleSet.rules).toEqual([rule1, rule2]);
		});

		it('should ignore falsy values', function () {
			var rule1 = new _ValidRuleSetChild2.default();
			var rule2 = new _ValidRuleSetChild2.default();
			var ruleSet = new _RuleSet2.default();
			ruleSet.add(rule1, undefined, false, 0, null, NaN, '', "", rule2);
			(0, _expect2.default)(ruleSet.rules).toEqual([rule1, rule2]);
		});

		it('should add a ruleset', function () {
			// First ruleset
			var rule = new _ValidRuleSetChild2.default();
			var firstRuleSet = new _RuleSet2.default();
			firstRuleSet.add(rule);
			(0, _expect2.default)(firstRuleSet.rules).toEqual([rule]);
			// Second ruleset
			var secondRuleSet = new _RuleSet2.default();
			secondRuleSet.add(firstRuleSet);
			(0, _expect2.default)(secondRuleSet.rules).toEqual(firstRuleSet.rules);
		});

		it('should add a ruleset to other rules', function () {
			// First ruleset
			var rule = new _ValidRuleSetChild2.default();
			var firstRuleSet = new _RuleSet2.default();
			firstRuleSet.add(rule);
			(0, _expect2.default)(firstRuleSet.rules).toEqual([rule]);
			// Second ruleset
			var secondRuleSet = new _RuleSet2.default();
			secondRuleSet.add(rule);
			secondRuleSet.add(firstRuleSet);
			(0, _expect2.default)(secondRuleSet.rules).toEqual([rule].concat(firstRuleSet.rules));
		});

		it('should throw if an invalid rule is passed in', function () {
			var ruleSet = new _RuleSet2.default();
			(0, _expect2.default)(function () {
				ruleSet.add('not a valid rule');
			}).toThrow(/Can\'t add this \^ to a RuleSet./);
		});
	});

	describe('flatten', function () {
		it('should flatten two rules', function () {
			var _expect$toEqual;

			var rule1 = { prop: 'something', value: 'else' };
			var rule2 = { prop: 'completely', value: 'different' };
			var ruleSet = new _RuleSet2.default();
			ruleSet.add(new _Rule2.default(rule1.prop, rule1.value), new _Rule2.default(rule2.prop, rule2.value));
			var result = ruleSet.flatten();
			(0, _expect2.default)(result.rules).toEqual((_expect$toEqual = {}, _defineProperty(_expect$toEqual, rule1.prop, rule1.value), _defineProperty(_expect$toEqual, rule2.prop, rule2.value), _expect$toEqual));
		});

		it('should override same properties', function () {
			var rule1 = { prop: 'something', value: 'else' };
			var rule2 = { prop: 'something', value: 'different' };
			var ruleSet = new _RuleSet2.default();
			ruleSet.add(new _Rule2.default(rule1.prop, rule1.value), new _Rule2.default(rule2.prop, rule2.value));
			var result = ruleSet.flatten();
			(0, _expect2.default)(result.rules).toEqual(_defineProperty({}, rule2.prop, rule2.value));
		});

		it('should flatten media queries', function () {
			var rule = { prop: 'something', value: 'else' };
			var query = 'max-width: 350px';
			// Media query
			var mediaQueryRuleSet = new _RuleSet2.default();
			mediaQueryRuleSet.add(new _Rule2.default(rule.prop, rule.value));
			var mediaQuery = new _MediaQuery2.default(query, mediaQueryRuleSet);
			// Ruleset
			var ruleSet = new _RuleSet2.default();
			ruleSet.add(mediaQuery);
			var result = ruleSet.flatten();
			(0, _expect2.default)(result.rules).toEqual(_defineProperty({}, '@media (' + query + ')', mediaQuery.flatten()));
		});

		it('should flatten a nested selector', function () {
			var rule = { prop: 'something', value: 'else' };
			// Nested selector
			var nestedRuleSet = new _RuleSet2.default();
			nestedRuleSet.add(new _Rule2.default(rule.prop, rule.value));
			var nestedSelector = new _NestedSelector2.default('div > p', nestedRuleSet);
			// Ruleset
			var ruleSet = new _RuleSet2.default();
			ruleSet.add(nestedSelector);
			var result = ruleSet.flatten();
			(0, _expect2.default)(result.fragments).toEqual([nestedSelector.flatten()]);
		});

		it('should flatten multiple nested selectors', function () {
			var rule = { prop: 'something', value: 'else' };
			// Nested selector
			var nestedRuleSet1 = new _RuleSet2.default();
			nestedRuleSet1.add(new _Rule2.default(rule.prop, rule.value));
			var nestedSelector1 = new _NestedSelector2.default('div > p', nestedRuleSet1);
			var nestedRuleSet2 = new _RuleSet2.default();
			nestedRuleSet2.add(new _Rule2.default(rule.prop, rule.value));
			var nestedSelector2 = new _NestedSelector2.default('div > p', nestedRuleSet2);
			// Ruleset
			var ruleSet = new _RuleSet2.default();
			ruleSet.add(nestedSelector1);
			ruleSet.add(nestedSelector2);
			var result = ruleSet.flatten();
			(0, _expect2.default)(result.fragments).toEqual([nestedSelector1.flatten(), nestedSelector2.flatten()]);
		});
	});
});